# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing
import uuid

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata


class ExportedCopilot(UniversalBaseModel):
    id: uuid.UUID = pydantic.Field()
    """
    The unique identifier of the copilot.
    """

    name: str = pydantic.Field()
    """
    The name of the deployed copilot version.
    """

    description: str = pydantic.Field()
    """
    The description of the deployed copilot version.
    """

    model_configuration: typing_extensions.Annotated[typing.Any, FieldMetadata(alias="modelConfiguration")] = (
        pydantic.Field()
    )
    """
    Model configuration including provider, model name, temperature, and other settings.
    """

    ai_endpoint_configuration: typing_extensions.Annotated[
        typing.Any, FieldMetadata(alias="aiEndpointConfiguration")
    ] = pydantic.Field()
    """
    AI endpoint configuration including base URL and authentication details.
    """

    tool_configurations: typing_extensions.Annotated[typing.Any, FieldMetadata(alias="toolConfigurations")] = (
        pydantic.Field()
    )
    """
    List of tool configurations available to the copilot.
    """

    inputs: typing.Any = pydantic.Field()
    """
    Input variable configurations for the copilot.
    """

    deployment_configuration: typing_extensions.Annotated[
        typing.Any, FieldMetadata(alias="deploymentConfiguration")
    ] = pydantic.Field()
    """
    Deployment settings and configuration.
    """

    agent_created_datetime: typing_extensions.Annotated[dt.datetime, FieldMetadata(alias="agentCreatedDatetime")] = (
        pydantic.Field()
    )
    """
    ISO 8601 timestamp when the copilot was originally created.
    """

    version_created_datetime: typing_extensions.Annotated[
        dt.datetime, FieldMetadata(alias="versionCreatedDatetime")
    ] = pydantic.Field()
    """
    ISO 8601 timestamp when the deployed version was created.
    """

    is_deployed: typing_extensions.Annotated[bool, FieldMetadata(alias="isDeployed")] = pydantic.Field()
    """
    Indicates if the copilot is currently deployed (always true for export results).
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing
import uuid
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.http_response import AsyncHttpResponse, HttpResponse
from ..core.pydantic_utilities import parse_obj_as
from ..core.request_options import RequestOptions
from ..core.serialization import convert_and_respect_annotation_metadata
from .types.document_collection_search_options import DocumentCollectionSearchOptions
from .types.search_document_collection_response import SearchDocumentCollectionResponse
from .types.single_field_filter import SingleFieldFilter

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawSearchClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def search_document_collection(
        self,
        *,
        collection_id: uuid.UUID,
        search_query: str,
        user_email: str,
        structured_query_filters: typing.Optional[typing.Sequence[SingleFieldFilter]] = OMIT,
        search_options: typing.Optional[DocumentCollectionSearchOptions] = OMIT,
        metadata_filter_expression: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[SearchDocumentCollectionResponse]:
        """
        Search across all documents in a document collection using the document metadata and contents.

        Parameters
        ----------
        collection_id : uuid.UUID

        search_query : str

        user_email : str
            The email of the user making the search request for permissions reduction.

        structured_query_filters : typing.Optional[typing.Sequence[SingleFieldFilter]]
            The structured query filters to apply to the search query.

        search_options : typing.Optional[DocumentCollectionSearchOptions]

        metadata_filter_expression : typing.Optional[str]
            Legacy metadata filter expression to apply to the search query. Use structuredQueryFilters instead.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[SearchDocumentCollectionResponse]
        """
        _response = self._client_wrapper.httpx_client.request(
            "v0/search/searchDocumentCollection",
            method="POST",
            json={
                "collectionId": collection_id,
                "searchQuery": search_query,
                "userEmail": user_email,
                "structuredQueryFilters": convert_and_respect_annotation_metadata(
                    object_=structured_query_filters, annotation=typing.Sequence[SingleFieldFilter], direction="write"
                ),
                "searchOptions": convert_and_respect_annotation_metadata(
                    object_=search_options, annotation=DocumentCollectionSearchOptions, direction="write"
                ),
                "metadataFilterExpression": metadata_filter_expression,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    SearchDocumentCollectionResponse,
                    parse_obj_as(
                        type_=SearchDocumentCollectionResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawSearchClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def search_document_collection(
        self,
        *,
        collection_id: uuid.UUID,
        search_query: str,
        user_email: str,
        structured_query_filters: typing.Optional[typing.Sequence[SingleFieldFilter]] = OMIT,
        search_options: typing.Optional[DocumentCollectionSearchOptions] = OMIT,
        metadata_filter_expression: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[SearchDocumentCollectionResponse]:
        """
        Search across all documents in a document collection using the document metadata and contents.

        Parameters
        ----------
        collection_id : uuid.UUID

        search_query : str

        user_email : str
            The email of the user making the search request for permissions reduction.

        structured_query_filters : typing.Optional[typing.Sequence[SingleFieldFilter]]
            The structured query filters to apply to the search query.

        search_options : typing.Optional[DocumentCollectionSearchOptions]

        metadata_filter_expression : typing.Optional[str]
            Legacy metadata filter expression to apply to the search query. Use structuredQueryFilters instead.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[SearchDocumentCollectionResponse]
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v0/search/searchDocumentCollection",
            method="POST",
            json={
                "collectionId": collection_id,
                "searchQuery": search_query,
                "userEmail": user_email,
                "structuredQueryFilters": convert_and_respect_annotation_metadata(
                    object_=structured_query_filters, annotation=typing.Sequence[SingleFieldFilter], direction="write"
                ),
                "searchOptions": convert_and_respect_annotation_metadata(
                    object_=search_options, annotation=DocumentCollectionSearchOptions, direction="write"
                ),
                "metadataFilterExpression": metadata_filter_expression,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    SearchDocumentCollectionResponse,
                    parse_obj_as(
                        type_=SearchDocumentCollectionResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata
from .resource_type import ResourceType


class ResourceIdentifier_ExternalResourceId(UniversalBaseModel):
    type: typing.Literal["external-resource-id"] = "external-resource-id"
    external_resource_id: typing_extensions.Annotated[str, FieldMetadata(alias="externalResourceId")]
    resource_type: typing_extensions.Annotated[ResourceType, FieldMetadata(alias="resourceType")]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class ResourceIdentifier_Url(UniversalBaseModel):
    type: typing.Literal["url"] = "url"
    url: str

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


ResourceIdentifier = typing_extensions.Annotated[
    typing.Union[ResourceIdentifier_ExternalResourceId, ResourceIdentifier_Url], pydantic.Field(discriminator="type")
]



# This file was auto-generated by Fern from our API Definition.

import typing
import uuid

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata
from .search_result_chunk import SearchResultChunk


class DocumentCollectionSearchResult(UniversalBaseModel):
    document_id: typing_extensions.Annotated[uuid.UUID, FieldMetadata(alias="documentId")]
    document_name: typing_extensions.Annotated[str, FieldMetadata(alias="documentName")]
    document_url: typing_extensions.Annotated[str, FieldMetadata(alias="documentUrl")]
    document_external_id: typing_extensions.Annotated[str, FieldMetadata(alias="documentExternalId")]
    document_metadata: typing_extensions.Annotated[typing.Dict[str, str], FieldMetadata(alias="documentMetadata")]
    chunks: typing.List[SearchResultChunk]
    merged_contents: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="mergedContents")] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata


class MongoSourceFieldsConfig(UniversalBaseModel):
    body: str
    source_name: typing_extensions.Annotated[str, FieldMetadata(alias="sourceName")]
    source_system_updated: typing_extensions.Annotated[
        typing.Optional[str], FieldMetadata(alias="sourceSystemUpdated")
    ] = None
    source_url: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="sourceUrl")] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing
import uuid

from ..common.types.collaborator import Collaborator
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from .raw_client import AsyncRawCopilotsClient, RawCopilotsClient
from .types.configuration import Configuration
from .types.create_conversation_response import CreateConversationResponse
from .types.create_copilot_response import CreateCopilotResponse
from .types.delete_copilot_response import DeleteCopilotResponse
from .types.export_copilots_response import ExportCopilotsResponse
from .types.input_variable import InputVariable
from .types.message_feedback import MessageFeedback
from .types.send_agent_message_response import SendAgentMessageResponse
from .types.streaming_chunk import StreamingChunk

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class CopilotsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawCopilotsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawCopilotsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawCopilotsClient
        """
        return self._raw_client

    def create_copilot(
        self,
        *,
        name: str,
        description: str,
        collaborators: typing.Sequence[Collaborator],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateCopilotResponse:
        """
        Create a new agent. The API key used will be added to the agent for future Requests

        Parameters
        ----------
        name : str
            A descriptive name for the agent.

        description : str
            An in depth name for the agent's function. Useful for routing requests to the right agent.

        collaborators : typing.Sequence[Collaborator]
            A list of collaborator emails and roles that will have access to the agent.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateCopilotResponse

        Examples
        --------
        from credal import CredalApi
        from credal.common import Collaborator

        client = CredalApi(
            api_key="YOUR_API_KEY",
        )
        client.copilots.create_copilot(
            name="Customer Agent",
            description="This agent is used to answer customer requests based on internal documentation.",
            collaborators=[
                Collaborator(
                    email="test@gmail.com",
                    role="editor",
                )
            ],
        )
        """
        _response = self._raw_client.create_copilot(
            name=name, description=description, collaborators=collaborators, request_options=request_options
        )
        return _response.data

    def create_conversation(
        self, *, agent_id: uuid.UUID, user_email: str, request_options: typing.Optional[RequestOptions] = None
    ) -> CreateConversationResponse:
        """
        OPTIONAL. Create a new conversation with the Agent. The conversation ID can be used in the `sendMessage` endpoint. The `sendMessage` endpoint automatically creates new conversations upon first request, but calling this endpoint can simplify certain use cases where it is helpful for the application to have the conversation ID before the first message is sent.

        Parameters
        ----------
        agent_id : uuid.UUID
            Credal-generated Agent ID to specify which agent to route the request to.

        user_email : str
            End-user for the conversation.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateConversationResponse

        Examples
        --------
        import uuid

        from credal import CredalApi

        client = CredalApi(
            api_key="YOUR_API_KEY",
        )
        client.copilots.create_conversation(
            agent_id=uuid.UUID(
                "82e4b12a-6990-45d4-8ebd-85c00e030c24",
            ),
            user_email="ravin@credal.ai",
        )
        """
        _response = self._raw_client.create_conversation(
            agent_id=agent_id, user_email=user_email, request_options=request_options
        )
        return _response.data

    def provide_message_feedback(
        self,
        *,
        agent_id: uuid.UUID,
        user_email: str,
        message_id: uuid.UUID,
        message_feedback: MessageFeedback,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Parameters
        ----------
        agent_id : uuid.UUID
            Credal-generated Agent ID to specify which agent to route the request to.

        user_email : str
            The user profile you want to use when providing feedback.

        message_id : uuid.UUID
            The message ID for which feedback is being provided.

        message_feedback : MessageFeedback
            The feedback provided by the user.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import uuid

        from credal import CredalApi
        from credal.copilots import MessageFeedback

        client = CredalApi(
            api_key="YOUR_API_KEY",
        )
        client.copilots.provide_message_feedback(
            user_email="ravin@credal.ai",
            message_id=uuid.UUID(
                "dd721cd8-4bf2-4b94-9869-258df3dab9dc",
            ),
            agent_id=uuid.UUID(
                "82e4b12a-6990-45d4-8ebd-85c00e030c24",
            ),
            message_feedback=MessageFeedback(
                feedback="NEGATIVE",
                suggested_answer="Yes, Credal is SOC 2 compliant.",
                descriptive_feedback="The response should be extremely clear and concise.",
            ),
        )
        """
        _response = self._raw_client.provide_message_feedback(
            agent_id=agent_id,
            user_email=user_email,
            message_id=message_id,
            message_feedback=message_feedback,
            request_options=request_options,
        )
        return _response.data

    def send_message(
        self,
        *,
        agent_id: uuid.UUID,
        message: str,
        user_email: str,
        conversation_id: typing.Optional[uuid.UUID] = OMIT,
        input_variables: typing.Optional[typing.Sequence[InputVariable]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SendAgentMessageResponse:
        """
        Parameters
        ----------
        agent_id : uuid.UUID
            Credal-generated Agent ID to specify which agent to route the request to.

        message : str
            The message you want to send to your agent.

        user_email : str
            The user profile you want to use when sending the message.

        conversation_id : typing.Optional[uuid.UUID]
            Credal-generated conversation ID for sending follow up messages. Conversation ID is returned after initial message. Optional, to be left off for first messages on new conversations.

        input_variables : typing.Optional[typing.Sequence[InputVariable]]
            Optional input variables to be used in the message. Map the name of the variable to a list of urls.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SendAgentMessageResponse

        Examples
        --------
        import uuid

        from credal import CredalApi
        from credal.copilots import InputVariable

        client = CredalApi(
            api_key="YOUR_API_KEY",
        )
        client.copilots.send_message(
            agent_id=uuid.UUID(
                "82e4b12a-6990-45d4-8ebd-85c00e030c24",
            ),
            message="Is Credal SOC 2 compliant?",
            user_email="ravin@credal.ai",
            input_variables=[
                InputVariable(
                    name="input1",
                    ids=[
                        uuid.UUID(
                            "82e4b12a-6990-45d4-8ebd-85c00e030c24",
                        )
                    ],
                ),
                InputVariable(
                    name="input2",
                    ids=[
                        uuid.UUID(
                            "82e4b12a-6990-45d4-8ebd-85c00e030c25",
                        ),
                        uuid.UUID(
                            "82e4b12a-6990-45d4-8ebd-85c00e030c26",
                        ),
                    ],
                ),
            ],
        )
        """
        _response = self._raw_client.send_message(
            agent_id=agent_id,
            message=message,
            user_email=user_email,
            conversation_id=conversation_id,
            input_variables=input_variables,
            request_options=request_options,
        )
        return _response.data

    def stream_message(
        self,
        *,
        copilot_id: uuid.UUID,
        message: str,
        email: str,
        conversation_id: typing.Optional[uuid.UUID] = OMIT,
        input_variables: typing.Optional[typing.Sequence[InputVariable]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.Iterator[StreamingChunk]:
        """
        This endpoint allows you to send a message to a specific agent and get the response back as a streamed set of Server-Sent Events.

        Parameters
        ----------
        copilot_id : uuid.UUID
            Credal-generated Agent ID to specify which agent to route the request to.

        message : str
            The message you want to send to your agent.

        email : str
            The user profile you want to use when sending the message.

        conversation_id : typing.Optional[uuid.UUID]
            Credal-generated conversation ID for sending follow up messages. Conversation ID is returned after initial message. Optional, to be left off for first messages on new conversations.

        input_variables : typing.Optional[typing.Sequence[InputVariable]]
            Optional input variables to be used in the message. Map the name of the variable to a list of urls.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Yields
        ------
        typing.Iterator[StreamingChunk]
            This endpoint returns a stream of server sent events. These can be in two formats - one is an initial event, followed by multiple data chunks, followed by a final chunk, or the other format is just one blocked event. See the examples for more details.

        Examples
        --------
        import uuid

        from credal import CredalApi
        from credal.copilots import InputVariable

        client = CredalApi(
            api_key="YOUR_API_KEY",
        )
        response = client.copilots.stream_message(
            copilot_id=uuid.UUID(
                "82e4b12a-6990-45d4-8ebd-85c00e030c24",
            ),
            message="Is Credal SOC 2 compliant?",
            email="ravin@credal.ai",
            input_variables=[
                InputVariable(
                    name="input1",
                    ids=[
                        uuid.UUID(
                            "82e4b12a-6990-45d4-8ebd-85c00e030c24",
                        )
                    ],
                ),
                InputVariable(
                    name="input2",
                    ids=[
                        uuid.UUID(
                            "82e4b12a-6990-45d4-8ebd-85c00e030c25",
                        ),
                        uuid.UUID(
                            "82e4b12a-6990-45d4-8ebd-85c00e030c26",
                        ),
                    ],
                ),
            ],
        )
        for chunk in response:
            yield chunk
        """
        with self._raw_client.stream_message(
            copilot_id=copilot_id,
            message=message,
            email=email,
            conversation_id=conversation_id,
            input_variables=input_variables,
            request_options=request_options,
        ) as r:
            yield from r.data

    def add_collection_to_copilot(
        self,
        *,
        copilot_id: uuid.UUID,
        collection_id: uuid.UUID,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Link a collection with a agent. The API Key used must be added to both the collection and the agent beforehand.

        Parameters
        ----------
        copilot_id : uuid.UUID
            Credal-generated Agent ID to add the collection to.

        collection_id : uuid.UUID
            Credal-generated collection ID to add.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import uuid

        from credal import CredalApi

        client = CredalApi(
            api_key="YOUR_API_KEY",
        )
        client.copilots.add_collection_to_copilot(
            copilot_id=uuid.UUID(
                "82e4b12a-6990-45d4-8ebd-85c00e030c24",
            ),
            collection_id=uuid.UUID(
                "def1055f-83c5-43d6-b558-f7a38e7b299e",
            ),
        )
        """
        _response = self._raw_client.add_collection_to_copilot(
            copilot_id=copilot_id, collection_id=collection_id, request_options=request_options
        )
        return _response.data

    def remove_collection_from_copilot(
        self,
        *,
        copilot_id: uuid.UUID,
        collection_id: uuid.UUID,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Unlink a collection with a agent. The API Key used must be added to both the collection and the agent beforehand.

        Parameters
        ----------
        copilot_id : uuid.UUID
            Credal-generated agent ID to add the collection to.

        collection_id : uuid.UUID
            Credal-generated collection ID to add.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import uuid

        from credal import CredalApi

        client = CredalApi(
            api_key="YOUR_API_KEY",
        )
        client.copilots.remove_collection_from_copilot(
            copilot_id=uuid.UUID(
                "82e4b12a-6990-45d4-8ebd-85c00e030c24",
            ),
            collection_id=uuid.UUID(
                "def1055f-83c5-43d6-b558-f7a38e7b299e",
            ),
        )
        """
        _response = self._raw_client.remove_collection_from_copilot(
            copilot_id=copilot_id, collection_id=collection_id, request_options=request_options
        )
        return _response.data

    def update_configuration(
        self,
        *,
        copilot_id: uuid.UUID,
        configuration: Configuration,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Update the configuration for a agent

        Parameters
        ----------
        copilot_id : uuid.UUID
            Credal-generated agent ID to add the collection to.

        configuration : Configuration

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import uuid

        from credal import CredalApi
        from credal.copilots import AiEndpointConfiguration, Configuration

        client = CredalApi(
            api_key="YOUR_API_KEY",
        )
        client.copilots.update_configuration(
            copilot_id=uuid.UUID(
                "82e4b12a-6990-45d4-8ebd-85c00e030c24",
            ),
            configuration=Configuration(
                name="Customer Agent",
                description="This agent is used to answer customer requests based on internal documentation.",
                prompt="You are a polite, helpful assistant used to answer customer requests.",
                ai_endpoint_configuration=AiEndpointConfiguration(
                    base_url="https://api.openai.com/v1/",
                    api_key="<YOUR_API_KEY_HERE>",
                ),
            ),
        )
        """
        _response = self._raw_client.update_configuration(
            copilot_id=copilot_id, configuration=configuration, request_options=request_options
        )
        return _response.data

    def delete_copilot(
        self, *, id: uuid.UUID, request_options: typing.Optional[RequestOptions] = None
    ) -> DeleteCopilotResponse:
        """
        Parameters
        ----------
        id : uuid.UUID
            Copilot ID

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DeleteCopilotResponse

        Examples
        --------
        import uuid

        from credal import CredalApi

        client = CredalApi(
            api_key="YOUR_API_KEY",
        )
        client.copilots.delete_copilot(
            id=uuid.UUID(
                "ac20e6ba-0bae-11ef-b25a-efca73df4c3a",
            ),
        )
        """
        _response = self._raw_client.delete_copilot(id=id, request_options=request_options)
        return _response.data

    def export(
        self,
        *,
        agent_created_from: typing.Optional[dt.datetime] = OMIT,
        agent_created_to: typing.Optional[dt.datetime] = OMIT,
        version_created_from: typing.Optional[dt.datetime] = OMIT,
        version_created_to: typing.Optional[dt.datetime] = OMIT,
        limit: typing.Optional[int] = OMIT,
        cursor: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ExportCopilotsResponse:
        """
        Export copilot configurations for backup or migration purposes.

        **IMPORTANT**: This endpoint requires:
        - Admin privileges
        - The 'ai-usage-analytics-log.export' scope on the API key

        Returns all deployed copilots with their full configuration including model settings, tools, and deployment details. Optional date filters can be applied to narrow down results.

        Parameters
        ----------
        agent_created_from : typing.Optional[dt.datetime]
            Filter copilots created on or after this datetime (ISO 8601 format).

        agent_created_to : typing.Optional[dt.datetime]
            Filter copilots created before or on this datetime (ISO 8601 format).

        version_created_from : typing.Optional[dt.datetime]
            Filter copilot versions created on or after this datetime (ISO 8601 format).

        version_created_to : typing.Optional[dt.datetime]
            Filter copilot versions created before or on this datetime (ISO 8601 format).

        limit : typing.Optional[int]
            Maximum number of copilots to return. Must be a positive integer with a maximum value of 1000. Defaults to 100.

        cursor : typing.Optional[str]
            Cursor for pagination. Use the cursor returned in the previous response to fetch the next page of results. If not provided, returns the first page.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ExportCopilotsResponse

        Examples
        --------
        import datetime

        from credal import CredalApi

        client = CredalApi(
            api_key="YOUR_API_KEY",
        )
        client.copilots.export(
            agent_created_from=datetime.datetime.fromisoformat(
                "2024-01-01 00:00:00+00:00",
            ),
            agent_created_to=datetime.datetime.fromisoformat(
                "2024-12-31 23:59:59+00:00",
            ),
        )
        """
        _response = self._raw_client.export(
            agent_created_from=agent_created_from,
            agent_created_to=agent_created_to,
            version_created_from=version_created_from,
            version_created_to=version_created_to,
            limit=limit,
            cursor=cursor,
            request_options=request_options,
        )
        return _response.data


class AsyncCopilotsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawCopilotsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawCopilotsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawCopilotsClient
        """
        return self._raw_client

    async def create_copilot(
        self,
        *,
        name: str,
        description: str,
        collaborators: typing.Sequence[Collaborator],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateCopilotResponse:
        """
        Create a new agent. The API key used will be added to the agent for future Requests

        Parameters
        ----------
        name : str
            A descriptive name for the agent.

        description : str
            An in depth name for the agent's function. Useful for routing requests to the right agent.

        collaborators : typing.Sequence[Collaborator]
            A list of collaborator emails and roles that will have access to the agent.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateCopilotResponse

        Examples
        --------
        import asyncio

        from credal import AsyncCredalApi
        from credal.common import Collaborator

        client = AsyncCredalApi(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.copilots.create_copilot(
                name="Customer Agent",
                description="This agent is used to answer customer requests based on internal documentation.",
                collaborators=[
                    Collaborator(
                        email="test@gmail.com",
                        role="editor",
                    )
                ],
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create_copilot(
            name=name, description=description, collaborators=collaborators, request_options=request_options
        )
        return _response.data

    async def create_conversation(
        self, *, agent_id: uuid.UUID, user_email: str, request_options: typing.Optional[RequestOptions] = None
    ) -> CreateConversationResponse:
        """
        OPTIONAL. Create a new conversation with the Agent. The conversation ID can be used in the `sendMessage` endpoint. The `sendMessage` endpoint automatically creates new conversations upon first request, but calling this endpoint can simplify certain use cases where it is helpful for the application to have the conversation ID before the first message is sent.

        Parameters
        ----------
        agent_id : uuid.UUID
            Credal-generated Agent ID to specify which agent to route the request to.

        user_email : str
            End-user for the conversation.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateConversationResponse

        Examples
        --------
        import asyncio
        import uuid

        from credal import AsyncCredalApi

        client = AsyncCredalApi(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.copilots.create_conversation(
                agent_id=uuid.UUID(
                    "82e4b12a-6990-45d4-8ebd-85c00e030c24",
                ),
                user_email="ravin@credal.ai",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create_conversation(
            agent_id=agent_id, user_email=user_email, request_options=request_options
        )
        return _response.data

    async def provide_message_feedback(
        self,
        *,
        agent_id: uuid.UUID,
        user_email: str,
        message_id: uuid.UUID,
        message_feedback: MessageFeedback,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Parameters
        ----------
        agent_id : uuid.UUID
            Credal-generated Agent ID to specify which agent to route the request to.

        user_email : str
            The user profile you want to use when providing feedback.

        message_id : uuid.UUID
            The message ID for which feedback is being provided.

        message_feedback : MessageFeedback
            The feedback provided by the user.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio
        import uuid

        from credal import AsyncCredalApi
        from credal.copilots import MessageFeedback

        client = AsyncCredalApi(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.copilots.provide_message_feedback(
                user_email="ravin@credal.ai",
                message_id=uuid.UUID(
                    "dd721cd8-4bf2-4b94-9869-258df3dab9dc",
                ),
                agent_id=uuid.UUID(
                    "82e4b12a-6990-45d4-8ebd-85c00e030c24",
                ),
                message_feedback=MessageFeedback(
                    feedback="NEGATIVE",
                    suggested_answer="Yes, Credal is SOC 2 compliant.",
                    descriptive_feedback="The response should be extremely clear and concise.",
                ),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.provide_message_feedback(
            agent_id=agent_id,
            user_email=user_email,
            message_id=message_id,
            message_feedback=message_feedback,
            request_options=request_options,
        )
        return _response.data

    async def send_message(
        self,
        *,
        agent_id: uuid.UUID,
        message: str,
        user_email: str,
        conversation_id: typing.Optional[uuid.UUID] = OMIT,
        input_variables: typing.Optional[typing.Sequence[InputVariable]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SendAgentMessageResponse:
        """
        Parameters
        ----------
        agent_id : uuid.UUID
            Credal-generated Agent ID to specify which agent to route the request to.

        message : str
            The message you want to send to your agent.

        user_email : str
            The user profile you want to use when sending the message.

        conversation_id : typing.Optional[uuid.UUID]
            Credal-generated conversation ID for sending follow up messages. Conversation ID is returned after initial message. Optional, to be left off for first messages on new conversations.

        input_variables : typing.Optional[typing.Sequence[InputVariable]]
            Optional input variables to be used in the message. Map the name of the variable to a list of urls.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SendAgentMessageResponse

        Examples
        --------
        import asyncio
        import uuid

        from credal import AsyncCredalApi
        from credal.copilots import InputVariable

        client = AsyncCredalApi(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.copilots.send_message(
                agent_id=uuid.UUID(
                    "82e4b12a-6990-45d4-8ebd-85c00e030c24",
                ),
                message="Is Credal SOC 2 compliant?",
                user_email="ravin@credal.ai",
                input_variables=[
                    InputVariable(
                        name="input1",
                        ids=[
                            uuid.UUID(
                                "82e4b12a-6990-45d4-8ebd-85c00e030c24",
                            )
                        ],
                    ),
                    InputVariable(
                        name="input2",
                        ids=[
                            uuid.UUID(
                                "82e4b12a-6990-45d4-8ebd-85c00e030c25",
                            ),
                            uuid.UUID(
                                "82e4b12a-6990-45d4-8ebd-85c00e030c26",
                            ),
                        ],
                    ),
                ],
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.send_message(
            agent_id=agent_id,
            message=message,
            user_email=user_email,
            conversation_id=conversation_id,
            input_variables=input_variables,
            request_options=request_options,
        )
        return _response.data

    async def stream_message(
        self,
        *,
        copilot_id: uuid.UUID,
        message: str,
        email: str,
        conversation_id: typing.Optional[uuid.UUID] = OMIT,
        input_variables: typing.Optional[typing.Sequence[InputVariable]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.AsyncIterator[StreamingChunk]:
        """
        This endpoint allows you to send a message to a specific agent and get the response back as a streamed set of Server-Sent Events.

        Parameters
        ----------
        copilot_id : uuid.UUID
            Credal-generated Agent ID to specify which agent to route the request to.

        message : str
            The message you want to send to your agent.

        email : str
            The user profile you want to use when sending the message.

        conversation_id : typing.Optional[uuid.UUID]
            Credal-generated conversation ID for sending follow up messages. Conversation ID is returned after initial message. Optional, to be left off for first messages on new conversations.

        input_variables : typing.Optional[typing.Sequence[InputVariable]]
            Optional input variables to be used in the message. Map the name of the variable to a list of urls.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Yields
        ------
        typing.AsyncIterator[StreamingChunk]
            This endpoint returns a stream of server sent events. These can be in two formats - one is an initial event, followed by multiple data chunks, followed by a final chunk, or the other format is just one blocked event. See the examples for more details.

        Examples
        --------
        import asyncio
        import uuid

        from credal import AsyncCredalApi
        from credal.copilots import InputVariable

        client = AsyncCredalApi(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            response = await client.copilots.stream_message(
                copilot_id=uuid.UUID(
                    "82e4b12a-6990-45d4-8ebd-85c00e030c24",
                ),
                message="Is Credal SOC 2 compliant?",
                email="ravin@credal.ai",
                input_variables=[
                    InputVariable(
                        name="input1",
                        ids=[
                            uuid.UUID(
                                "82e4b12a-6990-45d4-8ebd-85c00e030c24",
                            )
                        ],
                    ),
                    InputVariable(
                        name="input2",
                        ids=[
                            uuid.UUID(
                                "82e4b12a-6990-45d4-8ebd-85c00e030c25",
                            ),
                            uuid.UUID(
                                "82e4b12a-6990-45d4-8ebd-85c00e030c26",
                            ),
                        ],
                    ),
                ],
            )
            async for chunk in response:
                yield chunk


        asyncio.run(main())
        """
        async with self._raw_client.stream_message(
            copilot_id=copilot_id,
            message=message,
            email=email,
            conversation_id=conversation_id,
            input_variables=input_variables,
            request_options=request_options,
        ) as r:
            async for _chunk in r.data:
                yield _chunk

    async def add_collection_to_copilot(
        self,
        *,
        copilot_id: uuid.UUID,
        collection_id: uuid.UUID,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Link a collection with a agent. The API Key used must be added to both the collection and the agent beforehand.

        Parameters
        ----------
        copilot_id : uuid.UUID
            Credal-generated Agent ID to add the collection to.

        collection_id : uuid.UUID
            Credal-generated collection ID to add.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio
        import uuid

        from credal import AsyncCredalApi

        client = AsyncCredalApi(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.copilots.add_collection_to_copilot(
                copilot_id=uuid.UUID(
                    "82e4b12a-6990-45d4-8ebd-85c00e030c24",
                ),
                collection_id=uuid.UUID(
                    "def1055f-83c5-43d6-b558-f7a38e7b299e",
                ),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.add_collection_to_copilot(
            copilot_id=copilot_id, collection_id=collection_id, request_options=request_options
        )
        return _response.data

    async def remove_collection_from_copilot(
        self,
        *,
        copilot_id: uuid.UUID,
        collection_id: uuid.UUID,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Unlink a collection with a agent. The API Key used must be added to both the collection and the agent beforehand.

        Parameters
        ----------
        copilot_id : uuid.UUID
            Credal-generated agent ID to add the collection to.

        collection_id : uuid.UUID
            Credal-generated collection ID to add.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio
        import uuid

        from credal import AsyncCredalApi

        client = AsyncCredalApi(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.copilots.remove_collection_from_copilot(
                copilot_id=uuid.UUID(
                    "82e4b12a-6990-45d4-8ebd-85c00e030c24",
                ),
                collection_id=uuid.UUID(
                    "def1055f-83c5-43d6-b558-f7a38e7b299e",
                ),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.remove_collection_from_copilot(
            copilot_id=copilot_id, collection_id=collection_id, request_options=request_options
        )
        return _response.data

    async def update_configuration(
        self,
        *,
        copilot_id: uuid.UUID,
        configuration: Configuration,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Update the configuration for a agent

        Parameters
        ----------
        copilot_id : uuid.UUID
            Credal-generated agent ID to add the collection to.

        configuration : Configuration

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio
        import uuid

        from credal import AsyncCredalApi
        from credal.copilots import AiEndpointConfiguration, Configuration

        client = AsyncCredalApi(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.copilots.update_configuration(
                copilot_id=uuid.UUID(
                    "82e4b12a-6990-45d4-8ebd-85c00e030c24",
                ),
                configuration=Configuration(
                    name="Customer Agent",
                    description="This agent is used to answer customer requests based on internal documentation.",
                    prompt="You are a polite, helpful assistant used to answer customer requests.",
                    ai_endpoint_configuration=AiEndpointConfiguration(
                        base_url="https://api.openai.com/v1/",
                        api_key="<YOUR_API_KEY_HERE>",
                    ),
                ),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.update_configuration(
            copilot_id=copilot_id, configuration=configuration, request_options=request_options
        )
        return _response.data

    async def delete_copilot(
        self, *, id: uuid.UUID, request_options: typing.Optional[RequestOptions] = None
    ) -> DeleteCopilotResponse:
        """
        Parameters
        ----------
        id : uuid.UUID
            Copilot ID

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        DeleteCopilotResponse

        Examples
        --------
        import asyncio
        import uuid

        from credal import AsyncCredalApi

        client = AsyncCredalApi(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.copilots.delete_copilot(
                id=uuid.UUID(
                    "ac20e6ba-0bae-11ef-b25a-efca73df4c3a",
                ),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.delete_copilot(id=id, request_options=request_options)
        return _response.data

    async def export(
        self,
        *,
        agent_created_from: typing.Optional[dt.datetime] = OMIT,
        agent_created_to: typing.Optional[dt.datetime] = OMIT,
        version_created_from: typing.Optional[dt.datetime] = OMIT,
        version_created_to: typing.Optional[dt.datetime] = OMIT,
        limit: typing.Optional[int] = OMIT,
        cursor: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ExportCopilotsResponse:
        """
        Export copilot configurations for backup or migration purposes.

        **IMPORTANT**: This endpoint requires:
        - Admin privileges
        - The 'ai-usage-analytics-log.export' scope on the API key

        Returns all deployed copilots with their full configuration including model settings, tools, and deployment details. Optional date filters can be applied to narrow down results.

        Parameters
        ----------
        agent_created_from : typing.Optional[dt.datetime]
            Filter copilots created on or after this datetime (ISO 8601 format).

        agent_created_to : typing.Optional[dt.datetime]
            Filter copilots created before or on this datetime (ISO 8601 format).

        version_created_from : typing.Optional[dt.datetime]
            Filter copilot versions created on or after this datetime (ISO 8601 format).

        version_created_to : typing.Optional[dt.datetime]
            Filter copilot versions created before or on this datetime (ISO 8601 format).

        limit : typing.Optional[int]
            Maximum number of copilots to return. Must be a positive integer with a maximum value of 1000. Defaults to 100.

        cursor : typing.Optional[str]
            Cursor for pagination. Use the cursor returned in the previous response to fetch the next page of results. If not provided, returns the first page.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ExportCopilotsResponse

        Examples
        --------
        import asyncio
        import datetime

        from credal import AsyncCredalApi

        client = AsyncCredalApi(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.copilots.export(
                agent_created_from=datetime.datetime.fromisoformat(
                    "2024-01-01 00:00:00+00:00",
                ),
                agent_created_to=datetime.datetime.fromisoformat(
                    "2024-12-31 23:59:59+00:00",
                ),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.export(
            agent_created_from=agent_created_from,
            agent_created_to=agent_created_to,
            version_created_from=version_created_from,
            version_created_to=version_created_to,
            limit=limit,
            cursor=cursor,
            request_options=request_options,
        )
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from .. import core
from ..common.types.custom_metadata_value import CustomMetadataValue
from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.http_response import AsyncHttpResponse, HttpResponse
from ..core.pydantic_utilities import parse_obj_as
from ..core.request_options import RequestOptions
from ..core.serialization import convert_and_respect_annotation_metadata
from .types.document_metadata_patch import DocumentMetadataPatch
from .types.sync_source_by_url_response import SyncSourceByUrlResponse
from .types.upload_document_response import UploadDocumentResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawDocumentCatalogClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def upload_document_contents(
        self,
        *,
        document_name: str,
        document_contents: str,
        allowed_users_email_addresses: typing.Sequence[str],
        upload_as_user_email: str,
        document_external_id: str,
        document_external_url: typing.Optional[str] = OMIT,
        custom_metadata: typing.Optional[typing.Dict[str, CustomMetadataValue]] = OMIT,
        collection_id: typing.Optional[str] = OMIT,
        force_update: typing.Optional[bool] = OMIT,
        internal_public: typing.Optional[bool] = OMIT,
        await_vector_store_sync: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[UploadDocumentResponse]:
        """
        Parameters
        ----------
        document_name : str
            The name of the document you want to upload.

        document_contents : str
            The full LLM-formatted text contents of the document you want to upload.

        allowed_users_email_addresses : typing.Sequence[str]
            Users allowed to access the document. Unlike Credal's out of the box connectors which reconcile various permissions models from 3rd party software, for custom uploads the caller is responsible for specifying who can access the document and currently flattening groups if applicable. Documents can also be marked as internal public.

        upload_as_user_email : str
            [Legacy] The user on behalf of whom the document should be uploaded. In most cases, this can simply be the email of the developer making the API call. This field will be removed in the future in favor of purely specifying permissions via allowedUsersEmailAddresses.

        document_external_id : str
            The external ID of the document. This is typically the ID as it exists in its original external system. Uploads to the same external ID will update the document in Credal.

        document_external_url : typing.Optional[str]
            The external URL of the document you want to upload. If provided Credal will link to this URL.

        custom_metadata : typing.Optional[typing.Dict[str, CustomMetadataValue]]
            Optional JSON representing any custom metadata for this document

        collection_id : typing.Optional[str]
            If specified, the document will also be added to the provided document collection. This operation is eventually consistent, meaning the document does not immediately start appearing in searches of that collection due to an asynchronous embedding process. To achieve strong consistency use the `awaitVectorStoreSync` parameter.

        force_update : typing.Optional[bool]
            If specified, document contents will be re-uploaded and re-embedded even if the document already exists in Credal

        internal_public : typing.Optional[bool]
            If specified, document will be accessible to everyone within the organization of the uploader

        await_vector_store_sync : typing.Optional[bool]
            Document uploads are eventually consistent by default. If specified `true` the API will wait for the vector store to be updated before returning. This is useful if you want to ensure that the document is immediately searchable after this call returns.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[UploadDocumentResponse]
        """
        _response = self._client_wrapper.httpx_client.request(
            "v0/catalog/uploadDocumentContents",
            method="POST",
            json={
                "documentName": document_name,
                "documentContents": document_contents,
                "allowedUsersEmailAddresses": allowed_users_email_addresses,
                "uploadAsUserEmail": upload_as_user_email,
                "documentExternalId": document_external_id,
                "documentExternalUrl": document_external_url,
                "customMetadata": convert_and_respect_annotation_metadata(
                    object_=custom_metadata, annotation=typing.Dict[str, CustomMetadataValue], direction="write"
                ),
                "collectionId": collection_id,
                "forceUpdate": force_update,
                "internalPublic": internal_public,
                "awaitVectorStoreSync": await_vector_store_sync,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    UploadDocumentResponse,
                    parse_obj_as(
                        type_=UploadDocumentResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def upload_file(
        self,
        *,
        file: core.File,
        upload_as_user_email: str,
        document_external_id: str,
        document_name: typing.Optional[str] = OMIT,
        allowed_users_email_addresses: typing.Optional[str] = OMIT,
        document_external_url: typing.Optional[str] = OMIT,
        custom_metadata: typing.Optional[str] = OMIT,
        collection_id: typing.Optional[str] = OMIT,
        force_update: typing.Optional[str] = OMIT,
        internal_public: typing.Optional[str] = OMIT,
        source_system_updated: typing.Optional[str] = OMIT,
        await_vector_store_sync: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[UploadDocumentResponse]:
        """
        Upload a file (PDF, Word, Excel, CSV, PowerPoint) to Credal. Unlike uploadDocumentContents which requires pre-parsed text, this endpoint accepts actual file uploads and automatically parses them using Credal's parsing service.

        Parameters
        ----------
        file : core.File
            See core.File for more documentation

        upload_as_user_email : str
            [Legacy] The user on behalf of whom the document should be uploaded. In most cases, this can simply be the email of the developer making the API call. This field will be removed in the future in favor of purely specifying permissions via allowedUsersEmailAddresses.

        document_external_id : str
            The external ID of the document. This is typically the ID as it exists in its original external system. Uploads to the same external ID will update the document in Credal.

        document_name : typing.Optional[str]
            The name of the document you want to upload. If not provided, the original filename will be used.

        allowed_users_email_addresses : typing.Optional[str]
            Users allowed to access the document. Can be provided as a JSON array string (e.g., ["user1@example.com","user2@example.com"]) or comma-separated list (e.g., "user1@example.com,user2@example.com"). Unlike Credal's out of the box connectors which reconcile various permissions models from 3rd party software, for custom uploads the caller is responsible for specifying who can access the document and currently flattening groups if applicable. Documents can also be marked as internal public.

        document_external_url : typing.Optional[str]
            The external URL of the document you want to upload. If provided Credal will link to this URL.

        custom_metadata : typing.Optional[str]
            Optional JSON string representing any custom metadata for this document (e.g., '{"key1":"value1","key2":"value2"}').

        collection_id : typing.Optional[str]
            If specified, the document will also be added to the provided document collection. This operation is eventually consistent, meaning the document does not immediately start appearing in searches of that collection due to an asynchronous embedding process. To achieve strong consistency use the `awaitVectorStoreSync` parameter.

        force_update : typing.Optional[str]
            If set to "true", document contents will be re-uploaded and re-embedded even if the document already exists in Credal.

        internal_public : typing.Optional[str]
            If set to "true", document will be accessible to everyone within the organization of the uploader.

        source_system_updated : typing.Optional[str]
            ISO 8601 date string indicating when the document was last updated in the source system (e.g., "2025-11-03T21:15:00Z").

        await_vector_store_sync : typing.Optional[str]
            Document uploads are eventually consistent by default. If set to "true" the API will wait for the vector store to be updated before returning. This is useful if you want to ensure that the document is immediately searchable after this call returns.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[UploadDocumentResponse]
        """
        _response = self._client_wrapper.httpx_client.request(
            "v0/catalog/uploadFile",
            method="POST",
            data={
                "documentName": document_name,
                "uploadAsUserEmail": upload_as_user_email,
                "documentExternalId": document_external_id,
                "allowedUsersEmailAddresses": allowed_users_email_addresses,
                "documentExternalUrl": document_external_url,
                "customMetadata": custom_metadata,
                "collectionId": collection_id,
                "forceUpdate": force_update,
                "internalPublic": internal_public,
                "sourceSystemUpdated": source_system_updated,
                "awaitVectorStoreSync": await_vector_store_sync,
            },
            files={
                "file": file,
            },
            request_options=request_options,
            omit=OMIT,
            force_multipart=True,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    UploadDocumentResponse,
                    parse_obj_as(
                        type_=UploadDocumentResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def sync_source_by_url(
        self, *, upload_as_user_email: str, source_url: str, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[SyncSourceByUrlResponse]:
        """
        Sync a document from a source URL. Does not support recursive web search. Reach out to a Credal representative for access.

        Parameters
        ----------
        upload_as_user_email : str

        source_url : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[SyncSourceByUrlResponse]
        """
        _response = self._client_wrapper.httpx_client.request(
            "v0/catalog/syncSourceByUrl",
            method="POST",
            json={
                "uploadAsUserEmail": upload_as_user_email,
                "sourceUrl": source_url,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    SyncSourceByUrlResponse,
                    parse_obj_as(
                        type_=SyncSourceByUrlResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def metadata(
        self,
        *,
        sources: typing.Sequence[DocumentMetadataPatch],
        upload_as_user_email: str,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[None]:
        """
        Bulk patch metadata for documents, synced natively by Credal or manual API uploads

        Parameters
        ----------
        sources : typing.Sequence[DocumentMetadataPatch]

        upload_as_user_email : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[None]
        """
        _response = self._client_wrapper.httpx_client.request(
            "v0/catalog/metadata",
            method="PATCH",
            json={
                "sources": convert_and_respect_annotation_metadata(
                    object_=sources, annotation=typing.Sequence[DocumentMetadataPatch], direction="write"
                ),
                "uploadAsUserEmail": upload_as_user_email,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return HttpResponse(response=_response, data=None)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawDocumentCatalogClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def upload_document_contents(
        self,
        *,
        document_name: str,
        document_contents: str,
        allowed_users_email_addresses: typing.Sequence[str],
        upload_as_user_email: str,
        document_external_id: str,
        document_external_url: typing.Optional[str] = OMIT,
        custom_metadata: typing.Optional[typing.Dict[str, CustomMetadataValue]] = OMIT,
        collection_id: typing.Optional[str] = OMIT,
        force_update: typing.Optional[bool] = OMIT,
        internal_public: typing.Optional[bool] = OMIT,
        await_vector_store_sync: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[UploadDocumentResponse]:
        """
        Parameters
        ----------
        document_name : str
            The name of the document you want to upload.

        document_contents : str
            The full LLM-formatted text contents of the document you want to upload.

        allowed_users_email_addresses : typing.Sequence[str]
            Users allowed to access the document. Unlike Credal's out of the box connectors which reconcile various permissions models from 3rd party software, for custom uploads the caller is responsible for specifying who can access the document and currently flattening groups if applicable. Documents can also be marked as internal public.

        upload_as_user_email : str
            [Legacy] The user on behalf of whom the document should be uploaded. In most cases, this can simply be the email of the developer making the API call. This field will be removed in the future in favor of purely specifying permissions via allowedUsersEmailAddresses.

        document_external_id : str
            The external ID of the document. This is typically the ID as it exists in its original external system. Uploads to the same external ID will update the document in Credal.

        document_external_url : typing.Optional[str]
            The external URL of the document you want to upload. If provided Credal will link to this URL.

        custom_metadata : typing.Optional[typing.Dict[str, CustomMetadataValue]]
            Optional JSON representing any custom metadata for this document

        collection_id : typing.Optional[str]
            If specified, the document will also be added to the provided document collection. This operation is eventually consistent, meaning the document does not immediately start appearing in searches of that collection due to an asynchronous embedding process. To achieve strong consistency use the `awaitVectorStoreSync` parameter.

        force_update : typing.Optional[bool]
            If specified, document contents will be re-uploaded and re-embedded even if the document already exists in Credal

        internal_public : typing.Optional[bool]
            If specified, document will be accessible to everyone within the organization of the uploader

        await_vector_store_sync : typing.Optional[bool]
            Document uploads are eventually consistent by default. If specified `true` the API will wait for the vector store to be updated before returning. This is useful if you want to ensure that the document is immediately searchable after this call returns.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[UploadDocumentResponse]
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v0/catalog/uploadDocumentContents",
            method="POST",
            json={
                "documentName": document_name,
                "documentContents": document_contents,
                "allowedUsersEmailAddresses": allowed_users_email_addresses,
                "uploadAsUserEmail": upload_as_user_email,
                "documentExternalId": document_external_id,
                "documentExternalUrl": document_external_url,
                "customMetadata": convert_and_respect_annotation_metadata(
                    object_=custom_metadata, annotation=typing.Dict[str, CustomMetadataValue], direction="write"
                ),
                "collectionId": collection_id,
                "forceUpdate": force_update,
                "internalPublic": internal_public,
                "awaitVectorStoreSync": await_vector_store_sync,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    UploadDocumentResponse,
                    parse_obj_as(
                        type_=UploadDocumentResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def upload_file(
        self,
        *,
        file: core.File,
        upload_as_user_email: str,
        document_external_id: str,
        document_name: typing.Optional[str] = OMIT,
        allowed_users_email_addresses: typing.Optional[str] = OMIT,
        document_external_url: typing.Optional[str] = OMIT,
        custom_metadata: typing.Optional[str] = OMIT,
        collection_id: typing.Optional[str] = OMIT,
        force_update: typing.Optional[str] = OMIT,
        internal_public: typing.Optional[str] = OMIT,
        source_system_updated: typing.Optional[str] = OMIT,
        await_vector_store_sync: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[UploadDocumentResponse]:
        """
        Upload a file (PDF, Word, Excel, CSV, PowerPoint) to Credal. Unlike uploadDocumentContents which requires pre-parsed text, this endpoint accepts actual file uploads and automatically parses them using Credal's parsing service.

        Parameters
        ----------
        file : core.File
            See core.File for more documentation

        upload_as_user_email : str
            [Legacy] The user on behalf of whom the document should be uploaded. In most cases, this can simply be the email of the developer making the API call. This field will be removed in the future in favor of purely specifying permissions via allowedUsersEmailAddresses.

        document_external_id : str
            The external ID of the document. This is typically the ID as it exists in its original external system. Uploads to the same external ID will update the document in Credal.

        document_name : typing.Optional[str]
            The name of the document you want to upload. If not provided, the original filename will be used.

        allowed_users_email_addresses : typing.Optional[str]
            Users allowed to access the document. Can be provided as a JSON array string (e.g., ["user1@example.com","user2@example.com"]) or comma-separated list (e.g., "user1@example.com,user2@example.com"). Unlike Credal's out of the box connectors which reconcile various permissions models from 3rd party software, for custom uploads the caller is responsible for specifying who can access the document and currently flattening groups if applicable. Documents can also be marked as internal public.

        document_external_url : typing.Optional[str]
            The external URL of the document you want to upload. If provided Credal will link to this URL.

        custom_metadata : typing.Optional[str]
            Optional JSON string representing any custom metadata for this document (e.g., '{"key1":"value1","key2":"value2"}').

        collection_id : typing.Optional[str]
            If specified, the document will also be added to the provided document collection. This operation is eventually consistent, meaning the document does not immediately start appearing in searches of that collection due to an asynchronous embedding process. To achieve strong consistency use the `awaitVectorStoreSync` parameter.

        force_update : typing.Optional[str]
            If set to "true", document contents will be re-uploaded and re-embedded even if the document already exists in Credal.

        internal_public : typing.Optional[str]
            If set to "true", document will be accessible to everyone within the organization of the uploader.

        source_system_updated : typing.Optional[str]
            ISO 8601 date string indicating when the document was last updated in the source system (e.g., "2025-11-03T21:15:00Z").

        await_vector_store_sync : typing.Optional[str]
            Document uploads are eventually consistent by default. If set to "true" the API will wait for the vector store to be updated before returning. This is useful if you want to ensure that the document is immediately searchable after this call returns.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[UploadDocumentResponse]
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v0/catalog/uploadFile",
            method="POST",
            data={
                "documentName": document_name,
                "uploadAsUserEmail": upload_as_user_email,
                "documentExternalId": document_external_id,
                "allowedUsersEmailAddresses": allowed_users_email_addresses,
                "documentExternalUrl": document_external_url,
                "customMetadata": custom_metadata,
                "collectionId": collection_id,
                "forceUpdate": force_update,
                "internalPublic": internal_public,
                "sourceSystemUpdated": source_system_updated,
                "awaitVectorStoreSync": await_vector_store_sync,
            },
            files={
                "file": file,
            },
            request_options=request_options,
            omit=OMIT,
            force_multipart=True,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    UploadDocumentResponse,
                    parse_obj_as(
                        type_=UploadDocumentResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def sync_source_by_url(
        self, *, upload_as_user_email: str, source_url: str, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[SyncSourceByUrlResponse]:
        """
        Sync a document from a source URL. Does not support recursive web search. Reach out to a Credal representative for access.

        Parameters
        ----------
        upload_as_user_email : str

        source_url : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[SyncSourceByUrlResponse]
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v0/catalog/syncSourceByUrl",
            method="POST",
            json={
                "uploadAsUserEmail": upload_as_user_email,
                "sourceUrl": source_url,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    SyncSourceByUrlResponse,
                    parse_obj_as(
                        type_=SyncSourceByUrlResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def metadata(
        self,
        *,
        sources: typing.Sequence[DocumentMetadataPatch],
        upload_as_user_email: str,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[None]:
        """
        Bulk patch metadata for documents, synced natively by Credal or manual API uploads

        Parameters
        ----------
        sources : typing.Sequence[DocumentMetadataPatch]

        upload_as_user_email : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[None]
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v0/catalog/metadata",
            method="PATCH",
            json={
                "sources": convert_and_respect_annotation_metadata(
                    object_=sources, annotation=typing.Sequence[DocumentMetadataPatch], direction="write"
                ),
                "uploadAsUserEmail": upload_as_user_email,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return AsyncHttpResponse(response=_response, data=None)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

import typing
from importlib import import_module

if typing.TYPE_CHECKING:
    from .ai_endpoint_configuration import AiEndpointConfiguration
    from .blocked_chunk import BlockedChunk
    from .boolean_field_schema import BooleanFieldSchema
    from .collection_filtered_data import CollectionFilteredData
    from .configuration import Configuration
    from .create_conversation_response import CreateConversationResponse
    from .create_copilot_response import CreateCopilotResponse
    from .data_chunk import DataChunk
    from .data_filter import DataFilter
    from .datetime_field_schema import DatetimeFieldSchema
    from .delete_copilot_response import DeleteCopilotResponse
    from .end_of_message_chunk import EndOfMessageChunk
    from .error_chunk import ErrorChunk
    from .error_chunk_data import ErrorChunkData
    from .export_copilots_response import ExportCopilotsResponse
    from .exported_copilot import ExportedCopilot
    from .feedback_enum import FeedbackEnum
    from .filter import Filter, Filter_Boolean, Filter_Datetime, Filter_Number, Filter_String
    from .final_chunk import FinalChunk
    from .initial_chunk import InitialChunk
    from .input_variable import InputVariable
    from .inserted_audit_log import InsertedAuditLog
    from .message_blocked import MessageBlocked
    from .message_feedback import MessageFeedback
    from .message_reply import MessageReply
    from .number_field_schema import NumberFieldSchema
    from .referenced_source import ReferencedSource
    from .response_chunk import ResponseChunk
    from .send_agent_message_response import SendAgentMessageResponse
    from .send_message_response import (
        SendMessageResponse,
        SendMessageResponse_AiResponseResult,
        SendMessageResponse_BlockedResult,
    )
    from .streaming_chunk import (
        StreamingChunk,
        StreamingChunk_Blocked,
        StreamingChunk_DataChunk,
        StreamingChunk_EndOfMessage,
        StreamingChunk_ErrorChunk,
        StreamingChunk_FinalChunk,
        StreamingChunk_Initial,
    )
    from .string_field_schema import StringFieldSchema
    from .web_search_result import WebSearchResult
_dynamic_imports: typing.Dict[str, str] = {
    "AiEndpointConfiguration": ".ai_endpoint_configuration",
    "BlockedChunk": ".blocked_chunk",
    "BooleanFieldSchema": ".boolean_field_schema",
    "CollectionFilteredData": ".collection_filtered_data",
    "Configuration": ".configuration",
    "CreateConversationResponse": ".create_conversation_response",
    "CreateCopilotResponse": ".create_copilot_response",
    "DataChunk": ".data_chunk",
    "DataFilter": ".data_filter",
    "DatetimeFieldSchema": ".datetime_field_schema",
    "DeleteCopilotResponse": ".delete_copilot_response",
    "EndOfMessageChunk": ".end_of_message_chunk",
    "ErrorChunk": ".error_chunk",
    "ErrorChunkData": ".error_chunk_data",
    "ExportCopilotsResponse": ".export_copilots_response",
    "ExportedCopilot": ".exported_copilot",
    "FeedbackEnum": ".feedback_enum",
    "Filter": ".filter",
    "Filter_Boolean": ".filter",
    "Filter_Datetime": ".filter",
    "Filter_Number": ".filter",
    "Filter_String": ".filter",
    "FinalChunk": ".final_chunk",
    "InitialChunk": ".initial_chunk",
    "InputVariable": ".input_variable",
    "InsertedAuditLog": ".inserted_audit_log",
    "MessageBlocked": ".message_blocked",
    "MessageFeedback": ".message_feedback",
    "MessageReply": ".message_reply",
    "NumberFieldSchema": ".number_field_schema",
    "ReferencedSource": ".referenced_source",
    "ResponseChunk": ".response_chunk",
    "SendAgentMessageResponse": ".send_agent_message_response",
    "SendMessageResponse": ".send_message_response",
    "SendMessageResponse_AiResponseResult": ".send_message_response",
    "SendMessageResponse_BlockedResult": ".send_message_response",
    "StreamingChunk": ".streaming_chunk",
    "StreamingChunk_Blocked": ".streaming_chunk",
    "StreamingChunk_DataChunk": ".streaming_chunk",
    "StreamingChunk_EndOfMessage": ".streaming_chunk",
    "StreamingChunk_ErrorChunk": ".streaming_chunk",
    "StreamingChunk_FinalChunk": ".streaming_chunk",
    "StreamingChunk_Initial": ".streaming_chunk",
    "StringFieldSchema": ".string_field_schema",
    "WebSearchResult": ".web_search_result",
}


def __getattr__(attr_name: str) -> typing.Any:
    module_name = _dynamic_imports.get(attr_name)
    if module_name is None:
        raise AttributeError(f"No {attr_name} found in _dynamic_imports for module name -> {__name__}")
    try:
        module = import_module(module_name, __package__)
        if module_name == f".{attr_name}":
            return module
        else:
            return getattr(module, attr_name)
    except ImportError as e:
        raise ImportError(f"Failed to import {attr_name} from {module_name}: {e}") from e
    except AttributeError as e:
        raise AttributeError(f"Failed to get {attr_name} from {module_name}: {e}") from e


def __dir__():
    lazy_attrs = list(_dynamic_imports.keys())
    return sorted(lazy_attrs)


__all__ = [
    "AiEndpointConfiguration",
    "BlockedChunk",
    "BooleanFieldSchema",
    "CollectionFilteredData",
    "Configuration",
    "CreateConversationResponse",
    "CreateCopilotResponse",
    "DataChunk",
    "DataFilter",
    "DatetimeFieldSchema",
    "DeleteCopilotResponse",
    "EndOfMessageChunk",
    "ErrorChunk",
    "ErrorChunkData",
    "ExportCopilotsResponse",
    "ExportedCopilot",
    "FeedbackEnum",
    "Filter",
    "Filter_Boolean",
    "Filter_Datetime",
    "Filter_Number",
    "Filter_String",
    "FinalChunk",
    "InitialChunk",
    "InputVariable",
    "InsertedAuditLog",
    "MessageBlocked",
    "MessageFeedback",
    "MessageReply",
    "NumberFieldSchema",
    "ReferencedSource",
    "ResponseChunk",
    "SendAgentMessageResponse",
    "SendMessageResponse",
    "SendMessageResponse_AiResponseResult",
    "SendMessageResponse_BlockedResult",
    "StreamingChunk",
    "StreamingChunk_Blocked",
    "StreamingChunk_DataChunk",
    "StreamingChunk_EndOfMessage",
    "StreamingChunk_ErrorChunk",
    "StreamingChunk_FinalChunk",
    "StreamingChunk_Initial",
    "StringFieldSchema",
    "WebSearchResult",
]



# This file was auto-generated by Fern from our API Definition.

import typing

from .. import core
from ..common.types.custom_metadata_value import CustomMetadataValue
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from .raw_client import AsyncRawDocumentCatalogClient, RawDocumentCatalogClient
from .types.document_metadata_patch import DocumentMetadataPatch
from .types.sync_source_by_url_response import SyncSourceByUrlResponse
from .types.upload_document_response import UploadDocumentResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class DocumentCatalogClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawDocumentCatalogClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawDocumentCatalogClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawDocumentCatalogClient
        """
        return self._raw_client

    def upload_document_contents(
        self,
        *,
        document_name: str,
        document_contents: str,
        allowed_users_email_addresses: typing.Sequence[str],
        upload_as_user_email: str,
        document_external_id: str,
        document_external_url: typing.Optional[str] = OMIT,
        custom_metadata: typing.Optional[typing.Dict[str, CustomMetadataValue]] = OMIT,
        collection_id: typing.Optional[str] = OMIT,
        force_update: typing.Optional[bool] = OMIT,
        internal_public: typing.Optional[bool] = OMIT,
        await_vector_store_sync: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UploadDocumentResponse:
        """
        Parameters
        ----------
        document_name : str
            The name of the document you want to upload.

        document_contents : str
            The full LLM-formatted text contents of the document you want to upload.

        allowed_users_email_addresses : typing.Sequence[str]
            Users allowed to access the document. Unlike Credal's out of the box connectors which reconcile various permissions models from 3rd party software, for custom uploads the caller is responsible for specifying who can access the document and currently flattening groups if applicable. Documents can also be marked as internal public.

        upload_as_user_email : str
            [Legacy] The user on behalf of whom the document should be uploaded. In most cases, this can simply be the email of the developer making the API call. This field will be removed in the future in favor of purely specifying permissions via allowedUsersEmailAddresses.

        document_external_id : str
            The external ID of the document. This is typically the ID as it exists in its original external system. Uploads to the same external ID will update the document in Credal.

        document_external_url : typing.Optional[str]
            The external URL of the document you want to upload. If provided Credal will link to this URL.

        custom_metadata : typing.Optional[typing.Dict[str, CustomMetadataValue]]
            Optional JSON representing any custom metadata for this document

        collection_id : typing.Optional[str]
            If specified, the document will also be added to the provided document collection. This operation is eventually consistent, meaning the document does not immediately start appearing in searches of that collection due to an asynchronous embedding process. To achieve strong consistency use the `awaitVectorStoreSync` parameter.

        force_update : typing.Optional[bool]
            If specified, document contents will be re-uploaded and re-embedded even if the document already exists in Credal

        internal_public : typing.Optional[bool]
            If specified, document will be accessible to everyone within the organization of the uploader

        await_vector_store_sync : typing.Optional[bool]
            Document uploads are eventually consistent by default. If specified `true` the API will wait for the vector store to be updated before returning. This is useful if you want to ensure that the document is immediately searchable after this call returns.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UploadDocumentResponse

        Examples
        --------
        from credal import CredalApi

        client = CredalApi(
            api_key="YOUR_API_KEY",
        )
        client.document_catalog.upload_document_contents(
            document_name="My Document",
            document_contents="Lorem ipsum...",
            document_external_id="73eead26-d124-4940-b329-5f068a0a8db9",
            allowed_users_email_addresses=["jack@credal.ai", "ravin@credal.ai"],
            upload_as_user_email="jack@credal.ai",
        )
        """
        _response = self._raw_client.upload_document_contents(
            document_name=document_name,
            document_contents=document_contents,
            allowed_users_email_addresses=allowed_users_email_addresses,
            upload_as_user_email=upload_as_user_email,
            document_external_id=document_external_id,
            document_external_url=document_external_url,
            custom_metadata=custom_metadata,
            collection_id=collection_id,
            force_update=force_update,
            internal_public=internal_public,
            await_vector_store_sync=await_vector_store_sync,
            request_options=request_options,
        )
        return _response.data

    def upload_file(
        self,
        *,
        file: core.File,
        upload_as_user_email: str,
        document_external_id: str,
        document_name: typing.Optional[str] = OMIT,
        allowed_users_email_addresses: typing.Optional[str] = OMIT,
        document_external_url: typing.Optional[str] = OMIT,
        custom_metadata: typing.Optional[str] = OMIT,
        collection_id: typing.Optional[str] = OMIT,
        force_update: typing.Optional[str] = OMIT,
        internal_public: typing.Optional[str] = OMIT,
        source_system_updated: typing.Optional[str] = OMIT,
        await_vector_store_sync: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UploadDocumentResponse:
        """
        Upload a file (PDF, Word, Excel, CSV, PowerPoint) to Credal. Unlike uploadDocumentContents which requires pre-parsed text, this endpoint accepts actual file uploads and automatically parses them using Credal's parsing service.

        Parameters
        ----------
        file : core.File
            See core.File for more documentation

        upload_as_user_email : str
            [Legacy] The user on behalf of whom the document should be uploaded. In most cases, this can simply be the email of the developer making the API call. This field will be removed in the future in favor of purely specifying permissions via allowedUsersEmailAddresses.

        document_external_id : str
            The external ID of the document. This is typically the ID as it exists in its original external system. Uploads to the same external ID will update the document in Credal.

        document_name : typing.Optional[str]
            The name of the document you want to upload. If not provided, the original filename will be used.

        allowed_users_email_addresses : typing.Optional[str]
            Users allowed to access the document. Can be provided as a JSON array string (e.g., ["user1@example.com","user2@example.com"]) or comma-separated list (e.g., "user1@example.com,user2@example.com"). Unlike Credal's out of the box connectors which reconcile various permissions models from 3rd party software, for custom uploads the caller is responsible for specifying who can access the document and currently flattening groups if applicable. Documents can also be marked as internal public.

        document_external_url : typing.Optional[str]
            The external URL of the document you want to upload. If provided Credal will link to this URL.

        custom_metadata : typing.Optional[str]
            Optional JSON string representing any custom metadata for this document (e.g., '{"key1":"value1","key2":"value2"}').

        collection_id : typing.Optional[str]
            If specified, the document will also be added to the provided document collection. This operation is eventually consistent, meaning the document does not immediately start appearing in searches of that collection due to an asynchronous embedding process. To achieve strong consistency use the `awaitVectorStoreSync` parameter.

        force_update : typing.Optional[str]
            If set to "true", document contents will be re-uploaded and re-embedded even if the document already exists in Credal.

        internal_public : typing.Optional[str]
            If set to "true", document will be accessible to everyone within the organization of the uploader.

        source_system_updated : typing.Optional[str]
            ISO 8601 date string indicating when the document was last updated in the source system (e.g., "2025-11-03T21:15:00Z").

        await_vector_store_sync : typing.Optional[str]
            Document uploads are eventually consistent by default. If set to "true" the API will wait for the vector store to be updated before returning. This is useful if you want to ensure that the document is immediately searchable after this call returns.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UploadDocumentResponse
        """
        _response = self._raw_client.upload_file(
            file=file,
            upload_as_user_email=upload_as_user_email,
            document_external_id=document_external_id,
            document_name=document_name,
            allowed_users_email_addresses=allowed_users_email_addresses,
            document_external_url=document_external_url,
            custom_metadata=custom_metadata,
            collection_id=collection_id,
            force_update=force_update,
            internal_public=internal_public,
            source_system_updated=source_system_updated,
            await_vector_store_sync=await_vector_store_sync,
            request_options=request_options,
        )
        return _response.data

    def sync_source_by_url(
        self, *, upload_as_user_email: str, source_url: str, request_options: typing.Optional[RequestOptions] = None
    ) -> SyncSourceByUrlResponse:
        """
        Sync a document from a source URL. Does not support recursive web search. Reach out to a Credal representative for access.

        Parameters
        ----------
        upload_as_user_email : str

        source_url : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SyncSourceByUrlResponse

        Examples
        --------
        from credal import CredalApi

        client = CredalApi(
            api_key="YOUR_API_KEY",
        )
        client.document_catalog.sync_source_by_url(
            source_url="https://drive.google.com/file/d/123456/view",
            upload_as_user_email="ria@credal.ai",
        )
        """
        _response = self._raw_client.sync_source_by_url(
            upload_as_user_email=upload_as_user_email, source_url=source_url, request_options=request_options
        )
        return _response.data

    def metadata(
        self,
        *,
        sources: typing.Sequence[DocumentMetadataPatch],
        upload_as_user_email: str,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Bulk patch metadata for documents, synced natively by Credal or manual API uploads

        Parameters
        ----------
        sources : typing.Sequence[DocumentMetadataPatch]

        upload_as_user_email : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from credal import CredalApi
        from credal.common import ResourceIdentifier_ExternalResourceId
        from credal.document_catalog import DocumentMetadataPatch

        client = CredalApi(
            api_key="YOUR_API_KEY",
        )
        client.document_catalog.metadata(
            sources=[
                DocumentMetadataPatch(
                    metadata={"Department": "HR", "Country": "United States"},
                    resource_identifier=ResourceIdentifier_ExternalResourceId(
                        external_resource_id="170NrBm0Do7gdzvr54UvyslPVWkQFOA0lgNycFmdZJQr",
                        resource_type="GOOGLE_DRIVE_ITEM",
                    ),
                ),
                DocumentMetadataPatch(
                    metadata={"Department": "Sales", "Vertical": "Healthcare"},
                    resource_identifier=ResourceIdentifier_ExternalResourceId(
                        external_resource_id="123456",
                        resource_type="ZENDESK_TICKET",
                    ),
                ),
            ],
            upload_as_user_email="ben@credal.ai",
        )
        """
        _response = self._raw_client.metadata(
            sources=sources, upload_as_user_email=upload_as_user_email, request_options=request_options
        )
        return _response.data


class AsyncDocumentCatalogClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawDocumentCatalogClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawDocumentCatalogClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawDocumentCatalogClient
        """
        return self._raw_client

    async def upload_document_contents(
        self,
        *,
        document_name: str,
        document_contents: str,
        allowed_users_email_addresses: typing.Sequence[str],
        upload_as_user_email: str,
        document_external_id: str,
        document_external_url: typing.Optional[str] = OMIT,
        custom_metadata: typing.Optional[typing.Dict[str, CustomMetadataValue]] = OMIT,
        collection_id: typing.Optional[str] = OMIT,
        force_update: typing.Optional[bool] = OMIT,
        internal_public: typing.Optional[bool] = OMIT,
        await_vector_store_sync: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UploadDocumentResponse:
        """
        Parameters
        ----------
        document_name : str
            The name of the document you want to upload.

        document_contents : str
            The full LLM-formatted text contents of the document you want to upload.

        allowed_users_email_addresses : typing.Sequence[str]
            Users allowed to access the document. Unlike Credal's out of the box connectors which reconcile various permissions models from 3rd party software, for custom uploads the caller is responsible for specifying who can access the document and currently flattening groups if applicable. Documents can also be marked as internal public.

        upload_as_user_email : str
            [Legacy] The user on behalf of whom the document should be uploaded. In most cases, this can simply be the email of the developer making the API call. This field will be removed in the future in favor of purely specifying permissions via allowedUsersEmailAddresses.

        document_external_id : str
            The external ID of the document. This is typically the ID as it exists in its original external system. Uploads to the same external ID will update the document in Credal.

        document_external_url : typing.Optional[str]
            The external URL of the document you want to upload. If provided Credal will link to this URL.

        custom_metadata : typing.Optional[typing.Dict[str, CustomMetadataValue]]
            Optional JSON representing any custom metadata for this document

        collection_id : typing.Optional[str]
            If specified, the document will also be added to the provided document collection. This operation is eventually consistent, meaning the document does not immediately start appearing in searches of that collection due to an asynchronous embedding process. To achieve strong consistency use the `awaitVectorStoreSync` parameter.

        force_update : typing.Optional[bool]
            If specified, document contents will be re-uploaded and re-embedded even if the document already exists in Credal

        internal_public : typing.Optional[bool]
            If specified, document will be accessible to everyone within the organization of the uploader

        await_vector_store_sync : typing.Optional[bool]
            Document uploads are eventually consistent by default. If specified `true` the API will wait for the vector store to be updated before returning. This is useful if you want to ensure that the document is immediately searchable after this call returns.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UploadDocumentResponse

        Examples
        --------
        import asyncio

        from credal import AsyncCredalApi

        client = AsyncCredalApi(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.document_catalog.upload_document_contents(
                document_name="My Document",
                document_contents="Lorem ipsum...",
                document_external_id="73eead26-d124-4940-b329-5f068a0a8db9",
                allowed_users_email_addresses=["jack@credal.ai", "ravin@credal.ai"],
                upload_as_user_email="jack@credal.ai",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.upload_document_contents(
            document_name=document_name,
            document_contents=document_contents,
            allowed_users_email_addresses=allowed_users_email_addresses,
            upload_as_user_email=upload_as_user_email,
            document_external_id=document_external_id,
            document_external_url=document_external_url,
            custom_metadata=custom_metadata,
            collection_id=collection_id,
            force_update=force_update,
            internal_public=internal_public,
            await_vector_store_sync=await_vector_store_sync,
            request_options=request_options,
        )
        return _response.data

    async def upload_file(
        self,
        *,
        file: core.File,
        upload_as_user_email: str,
        document_external_id: str,
        document_name: typing.Optional[str] = OMIT,
        allowed_users_email_addresses: typing.Optional[str] = OMIT,
        document_external_url: typing.Optional[str] = OMIT,
        custom_metadata: typing.Optional[str] = OMIT,
        collection_id: typing.Optional[str] = OMIT,
        force_update: typing.Optional[str] = OMIT,
        internal_public: typing.Optional[str] = OMIT,
        source_system_updated: typing.Optional[str] = OMIT,
        await_vector_store_sync: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UploadDocumentResponse:
        """
        Upload a file (PDF, Word, Excel, CSV, PowerPoint) to Credal. Unlike uploadDocumentContents which requires pre-parsed text, this endpoint accepts actual file uploads and automatically parses them using Credal's parsing service.

        Parameters
        ----------
        file : core.File
            See core.File for more documentation

        upload_as_user_email : str
            [Legacy] The user on behalf of whom the document should be uploaded. In most cases, this can simply be the email of the developer making the API call. This field will be removed in the future in favor of purely specifying permissions via allowedUsersEmailAddresses.

        document_external_id : str
            The external ID of the document. This is typically the ID as it exists in its original external system. Uploads to the same external ID will update the document in Credal.

        document_name : typing.Optional[str]
            The name of the document you want to upload. If not provided, the original filename will be used.

        allowed_users_email_addresses : typing.Optional[str]
            Users allowed to access the document. Can be provided as a JSON array string (e.g., ["user1@example.com","user2@example.com"]) or comma-separated list (e.g., "user1@example.com,user2@example.com"). Unlike Credal's out of the box connectors which reconcile various permissions models from 3rd party software, for custom uploads the caller is responsible for specifying who can access the document and currently flattening groups if applicable. Documents can also be marked as internal public.

        document_external_url : typing.Optional[str]
            The external URL of the document you want to upload. If provided Credal will link to this URL.

        custom_metadata : typing.Optional[str]
            Optional JSON string representing any custom metadata for this document (e.g., '{"key1":"value1","key2":"value2"}').

        collection_id : typing.Optional[str]
            If specified, the document will also be added to the provided document collection. This operation is eventually consistent, meaning the document does not immediately start appearing in searches of that collection due to an asynchronous embedding process. To achieve strong consistency use the `awaitVectorStoreSync` parameter.

        force_update : typing.Optional[str]
            If set to "true", document contents will be re-uploaded and re-embedded even if the document already exists in Credal.

        internal_public : typing.Optional[str]
            If set to "true", document will be accessible to everyone within the organization of the uploader.

        source_system_updated : typing.Optional[str]
            ISO 8601 date string indicating when the document was last updated in the source system (e.g., "2025-11-03T21:15:00Z").

        await_vector_store_sync : typing.Optional[str]
            Document uploads are eventually consistent by default. If set to "true" the API will wait for the vector store to be updated before returning. This is useful if you want to ensure that the document is immediately searchable after this call returns.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UploadDocumentResponse
        """
        _response = await self._raw_client.upload_file(
            file=file,
            upload_as_user_email=upload_as_user_email,
            document_external_id=document_external_id,
            document_name=document_name,
            allowed_users_email_addresses=allowed_users_email_addresses,
            document_external_url=document_external_url,
            custom_metadata=custom_metadata,
            collection_id=collection_id,
            force_update=force_update,
            internal_public=internal_public,
            source_system_updated=source_system_updated,
            await_vector_store_sync=await_vector_store_sync,
            request_options=request_options,
        )
        return _response.data

    async def sync_source_by_url(
        self, *, upload_as_user_email: str, source_url: str, request_options: typing.Optional[RequestOptions] = None
    ) -> SyncSourceByUrlResponse:
        """
        Sync a document from a source URL. Does not support recursive web search. Reach out to a Credal representative for access.

        Parameters
        ----------
        upload_as_user_email : str

        source_url : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SyncSourceByUrlResponse

        Examples
        --------
        import asyncio

        from credal import AsyncCredalApi

        client = AsyncCredalApi(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.document_catalog.sync_source_by_url(
                source_url="https://drive.google.com/file/d/123456/view",
                upload_as_user_email="ria@credal.ai",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.sync_source_by_url(
            upload_as_user_email=upload_as_user_email, source_url=source_url, request_options=request_options
        )
        return _response.data

    async def metadata(
        self,
        *,
        sources: typing.Sequence[DocumentMetadataPatch],
        upload_as_user_email: str,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Bulk patch metadata for documents, synced natively by Credal or manual API uploads

        Parameters
        ----------
        sources : typing.Sequence[DocumentMetadataPatch]

        upload_as_user_email : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from credal import AsyncCredalApi
        from credal.common import ResourceIdentifier_ExternalResourceId
        from credal.document_catalog import DocumentMetadataPatch

        client = AsyncCredalApi(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.document_catalog.metadata(
                sources=[
                    DocumentMetadataPatch(
                        metadata={"Department": "HR", "Country": "United States"},
                        resource_identifier=ResourceIdentifier_ExternalResourceId(
                            external_resource_id="170NrBm0Do7gdzvr54UvyslPVWkQFOA0lgNycFmdZJQr",
                            resource_type="GOOGLE_DRIVE_ITEM",
                        ),
                    ),
                    DocumentMetadataPatch(
                        metadata={"Department": "Sales", "Vertical": "Healthcare"},
                        resource_identifier=ResourceIdentifier_ExternalResourceId(
                            external_resource_id="123456",
                            resource_type="ZENDESK_TICKET",
                        ),
                    ),
                ],
                upload_as_user_email="ben@credal.ai",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.metadata(
            sources=sources, upload_as_user_email=upload_as_user_email, request_options=request_options
        )
        return _response.data



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

import typing
from importlib import import_module

if typing.TYPE_CHECKING:
    from .create_collection_response import CreateCollectionResponse
    from .delete_collection_response import DeleteCollectionResponse
    from .list_documents_in_collection_response import ListDocumentsInCollectionResponse
    from .mongo_collection_sync_config import MongoCollectionSyncConfig
    from .mongo_collection_sync_response import MongoCollectionSyncResponse
    from .mongo_source_fields_config import MongoSourceFieldsConfig
_dynamic_imports: typing.Dict[str, str] = {
    "CreateCollectionResponse": ".create_collection_response",
    "DeleteCollectionResponse": ".delete_collection_response",
    "ListDocumentsInCollectionResponse": ".list_documents_in_collection_response",
    "MongoCollectionSyncConfig": ".mongo_collection_sync_config",
    "MongoCollectionSyncResponse": ".mongo_collection_sync_response",
    "MongoSourceFieldsConfig": ".mongo_source_fields_config",
}


def __getattr__(attr_name: str) -> typing.Any:
    module_name = _dynamic_imports.get(attr_name)
    if module_name is None:
        raise AttributeError(f"No {attr_name} found in _dynamic_imports for module name -> {__name__}")
    try:
        module = import_module(module_name, __package__)
        if module_name == f".{attr_name}":
            return module
        else:
            return getattr(module, attr_name)
    except ImportError as e:
        raise ImportError(f"Failed to import {attr_name} from {module_name}: {e}") from e
    except AttributeError as e:
        raise AttributeError(f"Failed to get {attr_name} from {module_name}: {e}") from e


def __dir__():
    lazy_attrs = list(_dynamic_imports.keys())
    return sorted(lazy_attrs)


__all__ = [
    "CreateCollectionResponse",
    "DeleteCollectionResponse",
    "ListDocumentsInCollectionResponse",
    "MongoCollectionSyncConfig",
    "MongoCollectionSyncResponse",
    "MongoSourceFieldsConfig",
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata
from .mongo_source_fields_config import MongoSourceFieldsConfig


class MongoCollectionSyncConfig(UniversalBaseModel):
    sync_name: typing_extensions.Annotated[str, FieldMetadata(alias="syncName")]
    collection_name: typing_extensions.Annotated[str, FieldMetadata(alias="collectionName")]
    filter_expression: typing_extensions.Annotated[typing.Any, FieldMetadata(alias="filterExpression")]
    source_fields: typing_extensions.Annotated[MongoSourceFieldsConfig, FieldMetadata(alias="sourceFields")]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing
import uuid
from json.decoder import JSONDecodeError

from ..common.types.collaborator import Collaborator
from ..common.types.resource_identifier import ResourceIdentifier
from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.http_response import AsyncHttpResponse, HttpResponse
from ..core.pydantic_utilities import parse_obj_as
from ..core.request_options import RequestOptions
from ..core.serialization import convert_and_respect_annotation_metadata
from .types.create_collection_response import CreateCollectionResponse
from .types.delete_collection_response import DeleteCollectionResponse
from .types.list_documents_in_collection_response import ListDocumentsInCollectionResponse
from .types.mongo_collection_sync_config import MongoCollectionSyncConfig
from .types.mongo_collection_sync_response import MongoCollectionSyncResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawDocumentCollectionsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def add_documents_to_collection(
        self,
        *,
        collection_id: uuid.UUID,
        resource_identifiers: typing.Sequence[ResourceIdentifier],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[None]:
        """
        Add documents to a document collection. Note that the documents must already exist in the document catalog to use this endpoint. If you want to upload a new document to a collection, use the `uploadDocumentContents` endpoint.

        Parameters
        ----------
        collection_id : uuid.UUID
            The ID of the document collection you want to add to.

        resource_identifiers : typing.Sequence[ResourceIdentifier]
            The set of resource identifier for which you want to add to the collection.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[None]
        """
        _response = self._client_wrapper.httpx_client.request(
            "v0/documentCollections/addDocumentsToCollection",
            method="POST",
            json={
                "collectionId": collection_id,
                "resourceIdentifiers": convert_and_respect_annotation_metadata(
                    object_=resource_identifiers, annotation=typing.Sequence[ResourceIdentifier], direction="write"
                ),
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return HttpResponse(response=_response, data=None)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def remove_documents_from_collection(
        self,
        *,
        collection_id: uuid.UUID,
        resource_identifiers: typing.Sequence[ResourceIdentifier],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[None]:
        """
        Remove documents from a collection

        Parameters
        ----------
        collection_id : uuid.UUID
            The ID of the document collection you want to add to.

        resource_identifiers : typing.Sequence[ResourceIdentifier]
            The set of resource identifier for which you want to remove from the collection

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[None]
        """
        _response = self._client_wrapper.httpx_client.request(
            "v0/documentCollections/removeDocumentsFromCollection",
            method="DELETE",
            json={
                "collectionId": collection_id,
                "resourceIdentifiers": convert_and_respect_annotation_metadata(
                    object_=resource_identifiers, annotation=typing.Sequence[ResourceIdentifier], direction="write"
                ),
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return HttpResponse(response=_response, data=None)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def list_documents_in_collection(
        self, *, collection_id: uuid.UUID, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[ListDocumentsInCollectionResponse]:
        """
        List documents in a collection

        Parameters
        ----------
        collection_id : uuid.UUID
            The ID of the document collection to list documents from.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[ListDocumentsInCollectionResponse]
        """
        _response = self._client_wrapper.httpx_client.request(
            "v0/documentCollections/listDocumentsInCollection",
            method="GET",
            params={
                "collectionId": collection_id,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ListDocumentsInCollectionResponse,
                    parse_obj_as(
                        type_=ListDocumentsInCollectionResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def create_collection(
        self,
        *,
        name: str,
        description: str,
        collaborators: typing.Sequence[Collaborator],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[CreateCollectionResponse]:
        """
        Create a new collection. The API key used will be added to the collection for future Requests

        Parameters
        ----------
        name : str
            A descriptive name for the collection.

        description : str
            An in depth name for the agent's function. Useful for routing requests to the right agent.

        collaborators : typing.Sequence[Collaborator]
            A list of collaborator emails and roles that will have access to the agent.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[CreateCollectionResponse]
        """
        _response = self._client_wrapper.httpx_client.request(
            "v0/documentCollections/createCollection",
            method="POST",
            json={
                "name": name,
                "description": description,
                "collaborators": convert_and_respect_annotation_metadata(
                    object_=collaborators, annotation=typing.Sequence[Collaborator], direction="write"
                ),
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CreateCollectionResponse,
                    parse_obj_as(
                        type_=CreateCollectionResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def delete_collection(
        self, *, collection_id: uuid.UUID, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[DeleteCollectionResponse]:
        """
        Delete the collection.

        Parameters
        ----------
        collection_id : uuid.UUID

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[DeleteCollectionResponse]
        """
        _response = self._client_wrapper.httpx_client.request(
            "v0/documentCollections/deleteCollection",
            method="DELETE",
            json={
                "collectionId": collection_id,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    DeleteCollectionResponse,
                    parse_obj_as(
                        type_=DeleteCollectionResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def create_mongo_collection_sync(
        self,
        *,
        collection_id: uuid.UUID,
        mongo_uri: str,
        config: MongoCollectionSyncConfig,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[MongoCollectionSyncResponse]:
        """
        Credal lets you easily sync your MongoDB data for use in Collections and Agents. Create a new sync from a MongoDB collection to a Credal collection.

        Parameters
        ----------
        collection_id : uuid.UUID

        mongo_uri : str

        config : MongoCollectionSyncConfig

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[MongoCollectionSyncResponse]
        """
        _response = self._client_wrapper.httpx_client.request(
            "v0/documentCollections/mongodb/createMongoSync",
            method="POST",
            json={
                "collectionId": collection_id,
                "mongoURI": mongo_uri,
                "config": convert_and_respect_annotation_metadata(
                    object_=config, annotation=MongoCollectionSyncConfig, direction="write"
                ),
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    MongoCollectionSyncResponse,
                    parse_obj_as(
                        type_=MongoCollectionSyncResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def update_mongo_collection_sync(
        self,
        *,
        mongo_credential_id: uuid.UUID,
        mongo_uri: str,
        config: MongoCollectionSyncConfig,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[MongoCollectionSyncResponse]:
        """
        Credal lets you easily sync your MongoDB data for use in Collections and Agents. Update an existing sync from a MongoDB collection to a Credal collection via the `mongoCredentialId`, to disambiguate between multiple potential syncs to a given collection.

        Parameters
        ----------
        mongo_credential_id : uuid.UUID

        mongo_uri : str

        config : MongoCollectionSyncConfig

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[MongoCollectionSyncResponse]
        """
        _response = self._client_wrapper.httpx_client.request(
            "v0/documentCollections/mongodb/updateMongoSync",
            method="POST",
            json={
                "mongoCredentialId": mongo_credential_id,
                "mongoURI": mongo_uri,
                "config": convert_and_respect_annotation_metadata(
                    object_=config, annotation=MongoCollectionSyncConfig, direction="write"
                ),
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    MongoCollectionSyncResponse,
                    parse_obj_as(
                        type_=MongoCollectionSyncResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawDocumentCollectionsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def add_documents_to_collection(
        self,
        *,
        collection_id: uuid.UUID,
        resource_identifiers: typing.Sequence[ResourceIdentifier],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[None]:
        """
        Add documents to a document collection. Note that the documents must already exist in the document catalog to use this endpoint. If you want to upload a new document to a collection, use the `uploadDocumentContents` endpoint.

        Parameters
        ----------
        collection_id : uuid.UUID
            The ID of the document collection you want to add to.

        resource_identifiers : typing.Sequence[ResourceIdentifier]
            The set of resource identifier for which you want to add to the collection.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[None]
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v0/documentCollections/addDocumentsToCollection",
            method="POST",
            json={
                "collectionId": collection_id,
                "resourceIdentifiers": convert_and_respect_annotation_metadata(
                    object_=resource_identifiers, annotation=typing.Sequence[ResourceIdentifier], direction="write"
                ),
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return AsyncHttpResponse(response=_response, data=None)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def remove_documents_from_collection(
        self,
        *,
        collection_id: uuid.UUID,
        resource_identifiers: typing.Sequence[ResourceIdentifier],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[None]:
        """
        Remove documents from a collection

        Parameters
        ----------
        collection_id : uuid.UUID
            The ID of the document collection you want to add to.

        resource_identifiers : typing.Sequence[ResourceIdentifier]
            The set of resource identifier for which you want to remove from the collection

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[None]
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v0/documentCollections/removeDocumentsFromCollection",
            method="DELETE",
            json={
                "collectionId": collection_id,
                "resourceIdentifiers": convert_and_respect_annotation_metadata(
                    object_=resource_identifiers, annotation=typing.Sequence[ResourceIdentifier], direction="write"
                ),
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return AsyncHttpResponse(response=_response, data=None)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def list_documents_in_collection(
        self, *, collection_id: uuid.UUID, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[ListDocumentsInCollectionResponse]:
        """
        List documents in a collection

        Parameters
        ----------
        collection_id : uuid.UUID
            The ID of the document collection to list documents from.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[ListDocumentsInCollectionResponse]
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v0/documentCollections/listDocumentsInCollection",
            method="GET",
            params={
                "collectionId": collection_id,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ListDocumentsInCollectionResponse,
                    parse_obj_as(
                        type_=ListDocumentsInCollectionResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def create_collection(
        self,
        *,
        name: str,
        description: str,
        collaborators: typing.Sequence[Collaborator],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[CreateCollectionResponse]:
        """
        Create a new collection. The API key used will be added to the collection for future Requests

        Parameters
        ----------
        name : str
            A descriptive name for the collection.

        description : str
            An in depth name for the agent's function. Useful for routing requests to the right agent.

        collaborators : typing.Sequence[Collaborator]
            A list of collaborator emails and roles that will have access to the agent.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[CreateCollectionResponse]
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v0/documentCollections/createCollection",
            method="POST",
            json={
                "name": name,
                "description": description,
                "collaborators": convert_and_respect_annotation_metadata(
                    object_=collaborators, annotation=typing.Sequence[Collaborator], direction="write"
                ),
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CreateCollectionResponse,
                    parse_obj_as(
                        type_=CreateCollectionResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def delete_collection(
        self, *, collection_id: uuid.UUID, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[DeleteCollectionResponse]:
        """
        Delete the collection.

        Parameters
        ----------
        collection_id : uuid.UUID

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[DeleteCollectionResponse]
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v0/documentCollections/deleteCollection",
            method="DELETE",
            json={
                "collectionId": collection_id,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    DeleteCollectionResponse,
                    parse_obj_as(
                        type_=DeleteCollectionResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def create_mongo_collection_sync(
        self,
        *,
        collection_id: uuid.UUID,
        mongo_uri: str,
        config: MongoCollectionSyncConfig,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[MongoCollectionSyncResponse]:
        """
        Credal lets you easily sync your MongoDB data for use in Collections and Agents. Create a new sync from a MongoDB collection to a Credal collection.

        Parameters
        ----------
        collection_id : uuid.UUID

        mongo_uri : str

        config : MongoCollectionSyncConfig

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[MongoCollectionSyncResponse]
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v0/documentCollections/mongodb/createMongoSync",
            method="POST",
            json={
                "collectionId": collection_id,
                "mongoURI": mongo_uri,
                "config": convert_and_respect_annotation_metadata(
                    object_=config, annotation=MongoCollectionSyncConfig, direction="write"
                ),
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    MongoCollectionSyncResponse,
                    parse_obj_as(
                        type_=MongoCollectionSyncResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def update_mongo_collection_sync(
        self,
        *,
        mongo_credential_id: uuid.UUID,
        mongo_uri: str,
        config: MongoCollectionSyncConfig,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[MongoCollectionSyncResponse]:
        """
        Credal lets you easily sync your MongoDB data for use in Collections and Agents. Update an existing sync from a MongoDB collection to a Credal collection via the `mongoCredentialId`, to disambiguate between multiple potential syncs to a given collection.

        Parameters
        ----------
        mongo_credential_id : uuid.UUID

        mongo_uri : str

        config : MongoCollectionSyncConfig

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[MongoCollectionSyncResponse]
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v0/documentCollections/mongodb/updateMongoSync",
            method="POST",
            json={
                "mongoCredentialId": mongo_credential_id,
                "mongoURI": mongo_uri,
                "config": convert_and_respect_annotation_metadata(
                    object_=config, annotation=MongoCollectionSyncConfig, direction="write"
                ),
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    MongoCollectionSyncResponse,
                    parse_obj_as(
                        type_=MongoCollectionSyncResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)
